@@ -46,6 +46,9 @@ class FatIO(io.RawIOBase):
         #: Current cluster chain offset (in bytes)
         self.__coffpos = 0
 
+        if self.mode.truncate:
+            self.seek(0)
+            self.truncate()
         if self.mode.appending:
             self.seek(0, 2)
 
@@ -112,7 +115,8 @@ class FatIO(io.RawIOBase):
     def close(self) -> None:
         """Close open file handles assuming lock handle."""
         self.seek(0)
-        self.fs.flush_fat()
+        if self.mode.writing:
+            self.fs.flush_fat()
         super().close()
 
     def readable(self) -> bool:
@@ -210,7 +214,8 @@ class FatIO(io.RawIOBase):
             self.write(b'\0' * (size - self.dir_entry.get_size()))
             self.seek(cur_pos)
         elif size < self.dir_entry.get_size():
-            num_clusters = self.fs.calc_num_clusters(size)
+            # Always keep at least one cluster allocated
+            num_clusters = max(1, self.fs.calc_num_clusters(size))
             i = 0
             for c in self.fs.get_cluster_chain(self.dir_entry.get_cluster()):
                 i += 1
